Copyright Dr. H. K. D. H. Bhadeshia, Department of Materials Science and
C Metallurgy, University of Cambridge, Pembroke Street, Cambridge CB2 3Q
C U. K.  Telephone Cambridge 334301
C
C 6 May    1987
C As FINITE but reads in data from time not equal to zero
C Program using finite difference method for the solution of the problem
C of X enrichment of cementite during the ageing of bainitic steels
C
C
C EQFER=Equilibrium wt.% of X in ferrite at ageing temperature
C EQCEM=Equilibrium wt.% of X in cementite at ageing temperature
C EBAR=Average X wt.% in alloy
C FERS normalised concentration of X at ferrite surface
C CEMS normalised concentration of X at cementite surface
C TIMH time in hours
C KTEMP=Absolute temperature
C TCEM=Thickness of cementite in meters
C TFER= Thickness of ferrite in meters
C DFER=Diffusivity of X in ferrite
C DCEM=Diffusivity of X in cementite
C Concentrations normalized relative to average alloy concentration
C Dimension normalize relative to carbide particle thickness
C ICEM, IFER, J1 are the number of finite slices
C  for dimension and time respectively
C TIM = Time, in seconds
C J4, J5 are related to the soft impingement detection subroutine
C JTEST is to adjust mass balance condition once cementite has
C  achieved its equilibrium condition at its surface
C ICFER2 and ICCEM2 refer to the number slices printed in previous run
C FREQ = PRE-EXPONENTIAL FACTOR FOR DIFFUSION COEFFICIENT
C Q = ACTIVATION FREE ENERGY FOR DIFFUSION COEFFICIENT
C
C
C Typical data
C 838.15 0.5D-03 39.0D+00 2.5D+00 1.0D-07 2.0D-06 1000 5
C 0.0003D+00 2.0D+00 43.2 6 5  2 2
C 2.53D-04 240580  0 DIFFUSION DATA , JTEST
C       1  0.7012D+01  Cementite
C       2  0.6998D+01
C       3  0.6988D+01
C       4  0.6983D+01               Slice number, normalised conc.
C       5  0.6982D+01
C       1  0.2205D-01    Ferrite
C       2  0.4387D-01
C       3  0.6565D-01
C       4  0.8737D-01
C       5  0.1090D+00
C       6  0.1306D+00
C      End of data
C
C
      IMPLICIT NONE                         
      DOUBLE PRECISION CFER(1500,2), CCEM(20,2)
      DOUBLE PRECISION KTEMP,EQFER,EQCEM,EBAR,TCEM,TFER,TIMH,FREQ,
     &Q,RFER,A3,RCEM,DFER,DCEM,STCEM,STFER,DUM1,TIM,CEMM,FERR,FERS,
     &FER,XTFER,CEMS,DR,CEM,DUMMY,AVER,AFER,ACEM,DIFF,TIME,XTCEM
      INTEGER J1,ICEM,IFER2,ICEM2,J4,J5,JTEST,I11,IFER,I12,I13,J,
     &II,I
        READ(5,*)KTEMP,EQFER,EQCEM,EBAR,TCEM,TFER,J1,ICEM
        READ(5,*)CFER(1,1), CCEM(1,1), TIMH, IFER2, ICEM2, J4, J5
        READ(5,*)FREQ,Q,JTEST
        RFER=0.40D+00
        A3=100.0
        RCEM=RFER
        DFER=DIFF(KTEMP,Q,FREQ)
        DCEM=DIFF(KTEMP,Q,FREQ)
        STCEM=0.5D+00*TCEM/ICEM
        IFER=DINT(TFER/STCEM)
        STFER=TFER/IFER
        TIME=RFER*STFER*STFER/DFER
        DO 60 I11=1,ICEM2
        READ(5,*)DUM1,CCEM(I11,1)
60      CONTINUE
        DO 61 I12=1,IFER2
        READ(5,*)DUM1,CFER(I12,1)
61      CONTINUE
        ICEM2=ICEM2+1
        IFER2=IFER2+1
        DO 62 I12=ICEM2,ICEM
        CCEM(I12,1)=1.0
62      CONTINUE
        DO 63 I13=IFER2,IFER
        CFER(I13,1)=1.0
63      CONTINUE
      WRITE(6,28)DFER,DCEM,TFER,TCEM,EQFER,EQCEM,KTEMP,ICEM,IFER
     &,STCEM,STFER
C
      FERS=EQFER/EBAR
      DR=DFER/DCEM
      WRITE(6,12)CCEM(1,1),CFER(1,1)
C
C Finite difference analysis
C
      TIM=TIMH*3600D+00
      WRITE(6,34)
      DO 1 J=2,J1
       TIM=TIM+TIME
       TIMH=TIM/3600.0D+00
       CEMM=0.0D+00
       FERR=0.0D+00
C
C                    ****    Ferrite    ******
C
      DO 20 II=1,IFER
      IF(II .EQ. 1)GOTO 21
C Ensure reflection at last slice
      IF(II .EQ. IFER)GOTO 22
      CFER(II,2)=CFER(II,1)+RFER*(CFER(II-1,1)
     &-2.0D+00*CFER(II,1)+CFER(II+1,1))
      GOTO 23
C Note: surface concentration in ferrite is  at equilibrium unless
C       CEMS has reached its equilibrium level, and JTEST=1
21     IF(JTEST .EQ. 0)GOTO 30
       FERS=((CCEM(1,1)-CEMS)/DR)+CFER(1,1)
30    CFER(1,2)=CFER(1,1) + RFER*(FERS - 2.0D+00*CFER(1,1)
     & + CFER(2,1))
      GOTO 23
22    CFER(IFER,2)=CFER(IFER,1)+RFER*(CFER(IFER-1,1)
     &-2.0D+00*CFER(IFER,1)+CFER(IFER-1,1))
      CALL SOFT(CFER(IFER,2),1,J4,TIMH)
23    FER=CFER(II,2)*EBAR
      XTFER=STFER*II
      FERR=FER+FERR
      CFER(II,1)=CFER(II,2)
20      CONTINUE
C
C                    ******   Cementite  ********
      DO 2 I=1,ICEM
C
      IF(I .EQ. 1)GOTO 3
C Reflect at position of symmetry
      IF(I .EQ. ICEM) GOTO 4
      CCEM(I,2)=CCEM(I,1)+RCEM*(CCEM(I-1,1)
     &-2.0D+00*CCEM(I,1)+CCEM(I+1,1))
      GOTO 5
C Calculate surface concentration in cementite appropriate for mass bala
3     CEMS=DR*(CFER(1,1)-FERS)+CCEM(1,1)
      IF(CEMS .GT. (EQCEM/EBAR))GOTO 90
      GOTO 91
90    CEMS=EQCEM/EBAR
      JTEST=1
91    CCEM(1,2)=CCEM(1,1) + RCEM*(CEMS-2.0D+00*CCEM(1,1)
     & + CCEM(2,1))
      GOTO 5
4     CCEM(ICEM,2)=CCEM(ICEM,1)+RCEM*(CCEM(ICEM-1,1)
     &-2.0D+00*CCEM(ICEM,1)+CCEM(ICEM-1,1))
      CALL SOFT(CCEM(ICEM,2),2,J5,TIMH)
5     CEM=CCEM(I,2)*EBAR
      XTCEM=I*STCEM
      CEMM=CEM+CEMM
      CCEM(I,1)=CCEM(I,2)
2     CONTINUE
C

C
      CEMM=CEMM/ICEM
      FERR=FERR/IFER
      DUMMY=J/A3
      DUMMY=DINT(DUMMY)-DUMMY
      IF(DUMMY .NE. 0.0)GOTO 1
      AVER=(FERR*TFER + CEMM*0.5D+00*TCEM)/(TFER+0.5D+00*TCEM)
      AFER=FERS*EBAR
      ACEM=CEMS*EBAR
      WRITE(6,27)TIMH,CEMM,FERR,AVER,AFER,ACEM
1     CONTINUE
      WRITE(6,25)
      DO 44 I=1,ICEM
      IF(CCEM(I,2) .LT. 1.0001)GOTO 46
      WRITE(6,7) I,CCEM(I,2),CCEM(I,2)*EBAR
44    CONTINUE
46    WRITE(6,24)
      DO 45 J=1,IFER
      IF(CFER(J,2) .GT. 0.999) GOTO 47
      WRITE(6,7) J,CFER(J,2),CFER(J,2)*EBAR
45    CONTINUE
47    CONTINUE
29    FORMAT(' TIME, s =',D12.4,'  TIME, hrs. =',F12.3)
28    FORMAT(' Diffusion coefficient in ferrite, m**2/s = ',
     &D12.4/' Diffusion coefficient in cementite, m**2/s, = ',D12.4/
     & ' Half thickness of ferrite, m =',D12.4/
     & ' Thickness of cementite, m =',D12.4/
     & ' Eq. conc. of X at interface, in ferrite, wt.% = ',D12.4/
     & ' Eq. conc. of X at interface, in cementite, wt.% = ',D12.4/
     & ' Absolute Temperature = ', F8.2, ' ICEM, IFER = ',2I9/
     & ' STCEM (m) = ',D12.4, '  STFER (m) = ',D12.4//)
34    FORMAT('    HOURS    CEM   FERRITE  AVERAGE X  FERS   CEMS')
27    FORMAT(F8.1,F9.4, F9.4,F9.4,F9.4,2F9.4)
24    FORMAT('      No   Norm. Conc.  wt.%X in Ferrite')
25    FORMAT(' No   Norm. Conc.    wt.%X in Cementite')
7     FORMAT(I8,D12.4,2F10.4)
11    FORMAT('--------------------------------------------------------')
12    FORMAT(' Time 0, slice 1, cementite and ferrite norm conc ',
     &2F12.4/)
      STOP
      END
C **********************************************************************
C
      DOUBLE PRECISION FUNCTION DIFF(KTEMP,Q,FREQ)
      DOUBLE PRECISION KTEMP, R , Q, FREQ
      R=8.3143
      DIFF=FREQ*DEXP(-Q/(R*KTEMP))
C R = Universal Gas Constant, J/mol/K
C KTEMP = Absolute Temperature
      RETURN
      END
C
C
      SUBROUTINE SOFT(A,I,J,TIMH)
      DOUBLE PRECISION A,TIMH
      IF(J .GT. 1) GOTO 3

      IF(I .NE. 1)GOTO 1
      IF(A .LT. 0.99) GOTO 2
      GOTO 3
2     WRITE(6,10)TIMH
      J=3
10    FORMAT(' SOFT IMPINGEMENT IN FERRITE', F10.2,' hours')
      GOTO 3
1     IF(A .GT. 1.01D+00)GOTO 4
      GOTO 3
4     WRITE(6,11)TIMH
      J=3
11    FORMAT(' SOFT IMPINGEMENT IN CEMENTITE',F10.2,' hours')
3     RETURN
      END
